/* Autogenerated: src/ExtractionOCaml/unsaturated_solinas --static poly1305 5 '2^130 - 5' 32 carry_mul carry_square carry add sub opp selectznz to_bytes from_bytes */
/* curve description: poly1305 */
/* requested operations: carry_mul, carry_square, carry, add, sub, opp, selectznz, to_bytes, from_bytes */
/* n = 5 (from "5") */
/* s-c = 2^130 - [(1, 5)] (from "2^130 - 5") */
/* machine_wordsize = 32 (from "32") */

/* Computed values: */
/* carry_chain = [0, 1, 2, 3, 4, 0, 1] */

#include <stdint.h>
typedef unsigned char fiat_poly1305_uint1;
typedef signed char fiat_poly1305_int1;

#if (-1 & 3) != 3
#error "This code only works on a two's complement system"
#endif


/*
 * The function fiat_poly1305_addcarryx_u26 is an addition with carry.
 * Postconditions:
 *   out1 = (arg1 + arg2 + arg3) mod 2^26
 *   out2 = ⌊(arg1 + arg2 + arg3) / 2^26⌋
 *
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [0x0 ~> 0x3ffffff]
 *   arg3: [0x0 ~> 0x3ffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0x3ffffff]
 *   out2: [0x0 ~> 0x1]
 */
static void fiat_poly1305_addcarryx_u26(uint32_t* out1, fiat_poly1305_uint1* out2, fiat_poly1305_uint1 arg1, uint32_t arg2, uint32_t arg3) {
  uint32_t x1 = ((arg1 + arg2) + arg3);
  uint32_t x2 = (x1 & UINT32_C(0x3ffffff));
  fiat_poly1305_uint1 x3 = (fiat_poly1305_uint1)(x1 >> 26);
  *out1 = x2;
  *out2 = x3;
}

/*
 * The function fiat_poly1305_subborrowx_u26 is a subtraction with borrow.
 * Postconditions:
 *   out1 = (-arg1 + arg2 + -arg3) mod 2^26
 *   out2 = -⌊(-arg1 + arg2 + -arg3) / 2^26⌋
 *
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [0x0 ~> 0x3ffffff]
 *   arg3: [0x0 ~> 0x3ffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0x3ffffff]
 *   out2: [0x0 ~> 0x1]
 */
static void fiat_poly1305_subborrowx_u26(uint32_t* out1, fiat_poly1305_uint1* out2, fiat_poly1305_uint1 arg1, uint32_t arg2, uint32_t arg3) {
  int32_t x1 = ((int32_t)(arg2 - arg1) - (int32_t)arg3);
  fiat_poly1305_int1 x2 = (fiat_poly1305_int1)(x1 >> 26);
  uint32_t x3 = (x1 & UINT32_C(0x3ffffff));
  *out1 = x3;
  *out2 = (fiat_poly1305_uint1)(0x0 - x2);
}

/*
 * The function fiat_poly1305_cmovznz_u32 is a single-word conditional move.
 * Postconditions:
 *   out1 = (if arg1 = 0 then arg2 else arg3)
 *
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [0x0 ~> 0xffffffff]
 *   arg3: [0x0 ~> 0xffffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffff]
 */
static void fiat_poly1305_cmovznz_u32(uint32_t* out1, fiat_poly1305_uint1 arg1, uint32_t arg2, uint32_t arg3) {
  fiat_poly1305_uint1 x1 = (!(!arg1));
  uint32_t x2 = ((fiat_poly1305_int1)(0x0 - x1) & UINT32_C(0xffffffff));
  uint32_t x3 = ((x2 & arg3) | ((~x2) & arg2));
  *out1 = x3;
}

/*
 * The function fiat_poly1305_carry_mul multiplies two field elements and reduces the result.
 * Postconditions:
 *   eval out1 mod m = (eval arg1 * eval arg2) mod m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332]]
 *   arg2: [[0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666]]
 */
static void fiat_poly1305_carry_mul(uint32_t out1[5], const uint32_t arg1[5], const uint32_t arg2[5]) {
  uint64_t x1 = ((uint64_t)(arg1[4]) * ((arg2[4]) * 0x5));
  uint64_t x2 = ((uint64_t)(arg1[4]) * ((arg2[3]) * 0x5));
  uint64_t x3 = ((uint64_t)(arg1[4]) * ((arg2[2]) * 0x5));
  uint64_t x4 = ((uint64_t)(arg1[4]) * ((arg2[1]) * 0x5));
  uint64_t x5 = ((uint64_t)(arg1[3]) * ((arg2[4]) * 0x5));
  uint64_t x6 = ((uint64_t)(arg1[3]) * ((arg2[3]) * 0x5));
  uint64_t x7 = ((uint64_t)(arg1[3]) * ((arg2[2]) * 0x5));
  uint64_t x8 = ((uint64_t)(arg1[2]) * ((arg2[4]) * 0x5));
  uint64_t x9 = ((uint64_t)(arg1[2]) * ((arg2[3]) * 0x5));
  uint64_t x10 = ((uint64_t)(arg1[1]) * ((arg2[4]) * 0x5));
  uint64_t x11 = ((uint64_t)(arg1[4]) * (arg2[0]));
  uint64_t x12 = ((uint64_t)(arg1[3]) * (arg2[1]));
  uint64_t x13 = ((uint64_t)(arg1[3]) * (arg2[0]));
  uint64_t x14 = ((uint64_t)(arg1[2]) * (arg2[2]));
  uint64_t x15 = ((uint64_t)(arg1[2]) * (arg2[1]));
  uint64_t x16 = ((uint64_t)(arg1[2]) * (arg2[0]));
  uint64_t x17 = ((uint64_t)(arg1[1]) * (arg2[3]));
  uint64_t x18 = ((uint64_t)(arg1[1]) * (arg2[2]));
  uint64_t x19 = ((uint64_t)(arg1[1]) * (arg2[1]));
  uint64_t x20 = ((uint64_t)(arg1[1]) * (arg2[0]));
  uint64_t x21 = ((uint64_t)(arg1[0]) * (arg2[4]));
  uint64_t x22 = ((uint64_t)(arg1[0]) * (arg2[3]));
  uint64_t x23 = ((uint64_t)(arg1[0]) * (arg2[2]));
  uint64_t x24 = ((uint64_t)(arg1[0]) * (arg2[1]));
  uint64_t x25 = ((uint64_t)(arg1[0]) * (arg2[0]));
  uint64_t x26 = (x25 + (x10 + (x9 + (x7 + x4))));
  uint64_t x27 = (x26 >> 26);
  uint32_t x28 = (uint32_t)(x26 & UINT32_C(0x3ffffff));
  uint64_t x29 = (x21 + (x17 + (x14 + (x12 + x11))));
  uint64_t x30 = (x22 + (x18 + (x15 + (x13 + x1))));
  uint64_t x31 = (x23 + (x19 + (x16 + (x5 + x2))));
  uint64_t x32 = (x24 + (x20 + (x8 + (x6 + x3))));
  uint64_t x33 = (x27 + x32);
  uint64_t x34 = (x33 >> 26);
  uint32_t x35 = (uint32_t)(x33 & UINT32_C(0x3ffffff));
  uint64_t x36 = (x34 + x31);
  uint64_t x37 = (x36 >> 26);
  uint32_t x38 = (uint32_t)(x36 & UINT32_C(0x3ffffff));
  uint64_t x39 = (x37 + x30);
  uint64_t x40 = (x39 >> 26);
  uint32_t x41 = (uint32_t)(x39 & UINT32_C(0x3ffffff));
  uint64_t x42 = (x40 + x29);
  uint32_t x43 = (uint32_t)(x42 >> 26);
  uint32_t x44 = (uint32_t)(x42 & UINT32_C(0x3ffffff));
  uint64_t x45 = ((uint64_t)x43 * 0x5);
  uint64_t x46 = (x28 + x45);
  uint32_t x47 = (uint32_t)(x46 >> 26);
  uint32_t x48 = (uint32_t)(x46 & UINT32_C(0x3ffffff));
  uint32_t x49 = (x47 + x35);
  fiat_poly1305_uint1 x50 = (fiat_poly1305_uint1)(x49 >> 26);
  uint32_t x51 = (x49 & UINT32_C(0x3ffffff));
  uint32_t x52 = (x50 + x38);
  out1[0] = x48;
  out1[1] = x51;
  out1[2] = x52;
  out1[3] = x41;
  out1[4] = x44;
}

/*
 * The function fiat_poly1305_carry_square squares a field element and reduces the result.
 * Postconditions:
 *   eval out1 mod m = (eval arg1 * eval arg1) mod m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666]]
 */
static void fiat_poly1305_carry_square(uint32_t out1[5], const uint32_t arg1[5]) {
  uint32_t x1 = ((arg1[4]) * 0x5);
  uint32_t x2 = (x1 * 0x2);
  uint32_t x3 = ((arg1[4]) * 0x2);
  uint32_t x4 = ((arg1[3]) * 0x5);
  uint32_t x5 = (x4 * 0x2);
  uint32_t x6 = ((arg1[3]) * 0x2);
  uint32_t x7 = ((arg1[2]) * 0x2);
  uint32_t x8 = ((arg1[1]) * 0x2);
  uint64_t x9 = ((uint64_t)(arg1[4]) * x1);
  uint64_t x10 = ((uint64_t)(arg1[3]) * x2);
  uint64_t x11 = ((uint64_t)(arg1[3]) * x4);
  uint64_t x12 = ((uint64_t)(arg1[2]) * x2);
  uint64_t x13 = ((uint64_t)(arg1[2]) * x5);
  uint64_t x14 = ((uint64_t)(arg1[2]) * (arg1[2]));
  uint64_t x15 = ((uint64_t)(arg1[1]) * x2);
  uint64_t x16 = ((uint64_t)(arg1[1]) * x6);
  uint64_t x17 = ((uint64_t)(arg1[1]) * x7);
  uint64_t x18 = ((uint64_t)(arg1[1]) * (arg1[1]));
  uint64_t x19 = ((uint64_t)(arg1[0]) * x3);
  uint64_t x20 = ((uint64_t)(arg1[0]) * x6);
  uint64_t x21 = ((uint64_t)(arg1[0]) * x7);
  uint64_t x22 = ((uint64_t)(arg1[0]) * x8);
  uint64_t x23 = ((uint64_t)(arg1[0]) * (arg1[0]));
  uint64_t x24 = (x23 + (x15 + x13));
  uint64_t x25 = (x24 >> 26);
  uint32_t x26 = (uint32_t)(x24 & UINT32_C(0x3ffffff));
  uint64_t x27 = (x19 + (x16 + x14));
  uint64_t x28 = (x20 + (x17 + x9));
  uint64_t x29 = (x21 + (x18 + x10));
  uint64_t x30 = (x22 + (x12 + x11));
  uint64_t x31 = (x25 + x30);
  uint64_t x32 = (x31 >> 26);
  uint32_t x33 = (uint32_t)(x31 & UINT32_C(0x3ffffff));
  uint64_t x34 = (x32 + x29);
  uint64_t x35 = (x34 >> 26);
  uint32_t x36 = (uint32_t)(x34 & UINT32_C(0x3ffffff));
  uint64_t x37 = (x35 + x28);
  uint64_t x38 = (x37 >> 26);
  uint32_t x39 = (uint32_t)(x37 & UINT32_C(0x3ffffff));
  uint64_t x40 = (x38 + x27);
  uint32_t x41 = (uint32_t)(x40 >> 26);
  uint32_t x42 = (uint32_t)(x40 & UINT32_C(0x3ffffff));
  uint64_t x43 = ((uint64_t)x41 * 0x5);
  uint64_t x44 = (x26 + x43);
  uint32_t x45 = (uint32_t)(x44 >> 26);
  uint32_t x46 = (uint32_t)(x44 & UINT32_C(0x3ffffff));
  uint32_t x47 = (x45 + x33);
  fiat_poly1305_uint1 x48 = (fiat_poly1305_uint1)(x47 >> 26);
  uint32_t x49 = (x47 & UINT32_C(0x3ffffff));
  uint32_t x50 = (x48 + x36);
  out1[0] = x46;
  out1[1] = x49;
  out1[2] = x50;
  out1[3] = x39;
  out1[4] = x42;
}

/*
 * The function fiat_poly1305_carry reduces a field element.
 * Postconditions:
 *   eval out1 mod m = eval arg1 mod m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666]]
 */
static void fiat_poly1305_carry(uint32_t out1[5], const uint32_t arg1[5]) {
  uint32_t x1 = (arg1[0]);
  uint32_t x2 = ((x1 >> 26) + (arg1[1]));
  uint32_t x3 = ((x2 >> 26) + (arg1[2]));
  uint32_t x4 = ((x3 >> 26) + (arg1[3]));
  uint32_t x5 = ((x4 >> 26) + (arg1[4]));
  uint32_t x6 = ((x1 & UINT32_C(0x3ffffff)) + ((x5 >> 26) * 0x5));
  uint32_t x7 = ((fiat_poly1305_uint1)(x6 >> 26) + (x2 & UINT32_C(0x3ffffff)));
  uint32_t x8 = (x6 & UINT32_C(0x3ffffff));
  uint32_t x9 = (x7 & UINT32_C(0x3ffffff));
  uint32_t x10 = ((fiat_poly1305_uint1)(x7 >> 26) + (x3 & UINT32_C(0x3ffffff)));
  uint32_t x11 = (x4 & UINT32_C(0x3ffffff));
  uint32_t x12 = (x5 & UINT32_C(0x3ffffff));
  out1[0] = x8;
  out1[1] = x9;
  out1[2] = x10;
  out1[3] = x11;
  out1[4] = x12;
}

/*
 * The function fiat_poly1305_add adds two field elements.
 * Postconditions:
 *   eval out1 mod m = (eval arg1 + eval arg2) mod m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666]]
 *   arg2: [[0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332]]
 */
static void fiat_poly1305_add(uint32_t out1[5], const uint32_t arg1[5], const uint32_t arg2[5]) {
  uint32_t x1 = ((arg1[0]) + (arg2[0]));
  uint32_t x2 = ((arg1[1]) + (arg2[1]));
  uint32_t x3 = ((arg1[2]) + (arg2[2]));
  uint32_t x4 = ((arg1[3]) + (arg2[3]));
  uint32_t x5 = ((arg1[4]) + (arg2[4]));
  out1[0] = x1;
  out1[1] = x2;
  out1[2] = x3;
  out1[3] = x4;
  out1[4] = x5;
}

/*
 * The function fiat_poly1305_sub subtracts two field elements.
 * Postconditions:
 *   eval out1 mod m = (eval arg1 - eval arg2) mod m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666]]
 *   arg2: [[0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332]]
 */
static void fiat_poly1305_sub(uint32_t out1[5], const uint32_t arg1[5], const uint32_t arg2[5]) {
  uint32_t x1 = ((UINT32_C(0x7fffff6) + (arg1[0])) - (arg2[0]));
  uint32_t x2 = ((UINT32_C(0x7fffffe) + (arg1[1])) - (arg2[1]));
  uint32_t x3 = ((UINT32_C(0x7fffffe) + (arg1[2])) - (arg2[2]));
  uint32_t x4 = ((UINT32_C(0x7fffffe) + (arg1[3])) - (arg2[3]));
  uint32_t x5 = ((UINT32_C(0x7fffffe) + (arg1[4])) - (arg2[4]));
  out1[0] = x1;
  out1[1] = x2;
  out1[2] = x3;
  out1[3] = x4;
  out1[4] = x5;
}

/*
 * The function fiat_poly1305_opp negates a field element.
 * Postconditions:
 *   eval out1 mod m = -eval arg1 mod m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332], [0x0 ~> 0xd333332]]
 */
static void fiat_poly1305_opp(uint32_t out1[5], const uint32_t arg1[5]) {
  uint32_t x1 = (UINT32_C(0x7fffff6) - (arg1[0]));
  uint32_t x2 = (UINT32_C(0x7fffffe) - (arg1[1]));
  uint32_t x3 = (UINT32_C(0x7fffffe) - (arg1[2]));
  uint32_t x4 = (UINT32_C(0x7fffffe) - (arg1[3]));
  uint32_t x5 = (UINT32_C(0x7fffffe) - (arg1[4]));
  out1[0] = x1;
  out1[1] = x2;
  out1[2] = x3;
  out1[3] = x4;
  out1[4] = x5;
}

/*
 * The function fiat_poly1305_selectznz is a multi-limb conditional select.
 * Postconditions:
 *   eval out1 = (if arg1 = 0 then eval arg2 else eval arg3)
 *
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 *   arg3: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 */
static void fiat_poly1305_selectznz(uint32_t out1[5], fiat_poly1305_uint1 arg1, const uint32_t arg2[5], const uint32_t arg3[5]) {
  uint32_t x1;
  fiat_poly1305_cmovznz_u32(&x1, arg1, (arg2[0]), (arg3[0]));
  uint32_t x2;
  fiat_poly1305_cmovznz_u32(&x2, arg1, (arg2[1]), (arg3[1]));
  uint32_t x3;
  fiat_poly1305_cmovznz_u32(&x3, arg1, (arg2[2]), (arg3[2]));
  uint32_t x4;
  fiat_poly1305_cmovznz_u32(&x4, arg1, (arg2[3]), (arg3[3]));
  uint32_t x5;
  fiat_poly1305_cmovznz_u32(&x5, arg1, (arg2[4]), (arg3[4]));
  out1[0] = x1;
  out1[1] = x2;
  out1[2] = x3;
  out1[3] = x4;
  out1[4] = x5;
}

/*
 * The function fiat_poly1305_to_bytes serializes a field element to bytes in little-endian order.
 * Postconditions:
 *   out1 = map (λ x, ⌊((eval arg1 mod m) mod 2^(8 * (x + 1))) / 2^(8 * x)⌋) [0..16]
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0x3]]
 */
static void fiat_poly1305_to_bytes(uint8_t out1[17], const uint32_t arg1[5]) {
  uint32_t x1;
  fiat_poly1305_uint1 x2;
  fiat_poly1305_subborrowx_u26(&x1, &x2, 0x0, (arg1[0]), UINT32_C(0x3fffffb));
  uint32_t x3;
  fiat_poly1305_uint1 x4;
  fiat_poly1305_subborrowx_u26(&x3, &x4, x2, (arg1[1]), UINT32_C(0x3ffffff));
  uint32_t x5;
  fiat_poly1305_uint1 x6;
  fiat_poly1305_subborrowx_u26(&x5, &x6, x4, (arg1[2]), UINT32_C(0x3ffffff));
  uint32_t x7;
  fiat_poly1305_uint1 x8;
  fiat_poly1305_subborrowx_u26(&x7, &x8, x6, (arg1[3]), UINT32_C(0x3ffffff));
  uint32_t x9;
  fiat_poly1305_uint1 x10;
  fiat_poly1305_subborrowx_u26(&x9, &x10, x8, (arg1[4]), UINT32_C(0x3ffffff));
  uint32_t x11;
  fiat_poly1305_cmovznz_u32(&x11, x10, 0x0, UINT32_C(0xffffffff));
  uint32_t x12;
  fiat_poly1305_uint1 x13;
  fiat_poly1305_addcarryx_u26(&x12, &x13, 0x0, x1, (x11 & UINT32_C(0x3fffffb)));
  uint32_t x14;
  fiat_poly1305_uint1 x15;
  fiat_poly1305_addcarryx_u26(&x14, &x15, x13, x3, (x11 & UINT32_C(0x3ffffff)));
  uint32_t x16;
  fiat_poly1305_uint1 x17;
  fiat_poly1305_addcarryx_u26(&x16, &x17, x15, x5, (x11 & UINT32_C(0x3ffffff)));
  uint32_t x18;
  fiat_poly1305_uint1 x19;
  fiat_poly1305_addcarryx_u26(&x18, &x19, x17, x7, (x11 & UINT32_C(0x3ffffff)));
  uint32_t x20;
  fiat_poly1305_uint1 x21;
  fiat_poly1305_addcarryx_u26(&x20, &x21, x19, x9, (x11 & UINT32_C(0x3ffffff)));
  uint32_t x22 = (x18 << 6);
  uint32_t x23 = (x16 << 4);
  uint32_t x24 = (x14 << 2);
  uint32_t x25 = (x12 >> 8);
  uint8_t x26 = (uint8_t)(x12 & UINT8_C(0xff));
  uint32_t x27 = (x25 >> 8);
  uint8_t x28 = (uint8_t)(x25 & UINT8_C(0xff));
  uint8_t x29 = (uint8_t)(x27 >> 8);
  uint8_t x30 = (uint8_t)(x27 & UINT8_C(0xff));
  uint32_t x31 = (x29 + x24);
  uint32_t x32 = (x31 >> 8);
  uint8_t x33 = (uint8_t)(x31 & UINT8_C(0xff));
  uint32_t x34 = (x32 >> 8);
  uint8_t x35 = (uint8_t)(x32 & UINT8_C(0xff));
  uint8_t x36 = (uint8_t)(x34 >> 8);
  uint8_t x37 = (uint8_t)(x34 & UINT8_C(0xff));
  uint32_t x38 = (x36 + x23);
  uint32_t x39 = (x38 >> 8);
  uint8_t x40 = (uint8_t)(x38 & UINT8_C(0xff));
  uint32_t x41 = (x39 >> 8);
  uint8_t x42 = (uint8_t)(x39 & UINT8_C(0xff));
  uint8_t x43 = (uint8_t)(x41 >> 8);
  uint8_t x44 = (uint8_t)(x41 & UINT8_C(0xff));
  uint32_t x45 = (x43 + x22);
  uint32_t x46 = (x45 >> 8);
  uint8_t x47 = (uint8_t)(x45 & UINT8_C(0xff));
  uint32_t x48 = (x46 >> 8);
  uint8_t x49 = (uint8_t)(x46 & UINT8_C(0xff));
  uint8_t x50 = (uint8_t)(x48 >> 8);
  uint8_t x51 = (uint8_t)(x48 & UINT8_C(0xff));
  uint8_t x52 = (uint8_t)(x50 & UINT8_C(0xff));
  uint32_t x53 = (x20 >> 8);
  uint8_t x54 = (uint8_t)(x20 & UINT8_C(0xff));
  uint32_t x55 = (x53 >> 8);
  uint8_t x56 = (uint8_t)(x53 & UINT8_C(0xff));
  uint8_t x57 = (uint8_t)(x55 >> 8);
  uint8_t x58 = (uint8_t)(x55 & UINT8_C(0xff));
  out1[0] = x26;
  out1[1] = x28;
  out1[2] = x30;
  out1[3] = x33;
  out1[4] = x35;
  out1[5] = x37;
  out1[6] = x40;
  out1[7] = x42;
  out1[8] = x44;
  out1[9] = x47;
  out1[10] = x49;
  out1[11] = x51;
  out1[12] = x52;
  out1[13] = x54;
  out1[14] = x56;
  out1[15] = x58;
  out1[16] = x57;
}

/*
 * The function fiat_poly1305_from_bytes deserializes a field element from bytes in little-endian order.
 * Postconditions:
 *   eval out1 mod m = bytes_eval arg1 mod m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0x3]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666], [0x0 ~> 0x4666666]]
 */
static void fiat_poly1305_from_bytes(uint32_t out1[5], const uint8_t arg1[17]) {
  uint32_t x1 = ((uint32_t)(arg1[16]) << 24);
  uint32_t x2 = ((uint32_t)(arg1[15]) << 16);
  uint32_t x3 = ((uint32_t)(arg1[14]) << 8);
  uint8_t x4 = (arg1[13]);
  uint32_t x5 = ((uint32_t)(arg1[12]) << 18);
  uint32_t x6 = ((uint32_t)(arg1[11]) << 10);
  uint32_t x7 = ((uint32_t)(arg1[10]) << 2);
  uint32_t x8 = ((uint32_t)(arg1[9]) << 20);
  uint32_t x9 = ((uint32_t)(arg1[8]) << 12);
  uint32_t x10 = ((uint32_t)(arg1[7]) << 4);
  uint32_t x11 = ((uint32_t)(arg1[6]) << 22);
  uint32_t x12 = ((uint32_t)(arg1[5]) << 14);
  uint32_t x13 = ((uint32_t)(arg1[4]) << 6);
  uint32_t x14 = ((uint32_t)(arg1[3]) << 24);
  uint32_t x15 = ((uint32_t)(arg1[2]) << 16);
  uint32_t x16 = ((uint32_t)(arg1[1]) << 8);
  uint8_t x17 = (arg1[0]);
  uint32_t x18 = (x17 + (x16 + (x15 + x14)));
  uint8_t x19 = (uint8_t)(x18 >> 26);
  uint32_t x20 = (x18 & UINT32_C(0x3ffffff));
  uint32_t x21 = (x4 + (x3 + (x2 + x1)));
  uint32_t x22 = (x7 + (x6 + x5));
  uint32_t x23 = (x10 + (x9 + x8));
  uint32_t x24 = (x13 + (x12 + x11));
  uint32_t x25 = (x19 + x24);
  uint8_t x26 = (uint8_t)(x25 >> 26);
  uint32_t x27 = (x25 & UINT32_C(0x3ffffff));
  uint32_t x28 = (x26 + x23);
  uint8_t x29 = (uint8_t)(x28 >> 26);
  uint32_t x30 = (x28 & UINT32_C(0x3ffffff));
  uint32_t x31 = (x29 + x22);
  uint32_t x32 = (x31 & UINT32_C(0x3ffffff));
  out1[0] = x20;
  out1[1] = x27;
  out1[2] = x30;
  out1[3] = x32;
  out1[4] = x21;
}

